{% if version.locked_message %}<div class="cms-toolbar-item versioning-lock-message muted" style="margin-top: 12px; opacity:0.4;"><span class="cms-icon cms-icon-lock"></span> <small>{{ version.locked_message }}</small></div>{% endif %}
