# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-03 13:32
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("djangocms_versioning", "0004_auto_20180730_1135")]

    operations = [migrations.RemoveField(model_name="version", name="label")]
