{% extends "base.html" %}
{% load static i18n %}
{% block title %}{% translate 'Preview' %}{% endblock %}
{% block name %}preview{% endblock %}
{% block icon %}{% static 'djangocms_versioning/svg/preview.svg' %}{% endblock %}
