# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-10-05 14:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("djangocms_versioning", "0010_version_proxies")]

    operations = [
        migrations.AddField(
            model_name="version",
            name="number",
            field=models.CharField(max_length=11, null=True),
        )
    ]
