{% if menu %}<a href="#" class="cms-pagetree-dropdown-trigger js-cms-pagetree-dropdown-trigger" data-menu="{{ menu }}">{% endif %}
<span class="cms-tree-item-lang cms-pagetree-node-state cms-pagetree-node-state-{{ state }}" title="{{ description }}"></span>
{% if menu %}</a>{% endif %}

